﻿// 1. Use the AxImp utility to create the .Net wrappers for the  ShDocVw.dll. This will create 2 assemblies: shdocvw.dll and AxShDocVw.DLL which must be added as reference to your project
// 2. reference to Microsoft.mshtml - NET tab, Microsoft mshtml
// 3. Compile the file


using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Windows.Forms;
using System.Drawing.Imaging;
using System.Runtime.InteropServices;

// IE 
using AxSHDocVw; 
using SHDocVw;  
using mshtml;   


//This classes works as following:
//* creates invisible browser control
//* navigates to a url (or filepath)
//* waits for it to load
//* finds out the size of the page
//* resizes the control
//* captures the image
//* create several images of the size of 1 page
 public class Html2Image : System.Windows.Forms.UserControl
 {
		private AxSHDocVw.AxWebBrowser mIE;
 	
  // this method returns a list of PNG files
  // url: html file to load
  // pageLength: length in pixels of the pages. The html image will be splitted according to this value
  // outputDir: output directory for the png files
  // outputFile: output file names will be made of this prefix , the page number a the extension .png
  static public String[] toPNG(string url, int pageLength, string outputDir, string outputFile)
  {

   Html2Image html2Image = new Html2Image();
   html2Image.CreateControl();
   string[] f=html2Image.createFiles(url,pageLength,outputDir,outputFile);
   html2Image.Dispose();
  	
   return f;
  }


  private bool mDocComplete = false;
  private System.ComponentModel.Container 
  components = null;

  private Html2Image()
  {
   // setups our embedded ie control
   InitializeComponent();
  }

  private string[] createFiles(string url, int pageLength, string outputDir, string outputFile)
  {
   // Make sure the ie control is ready, and if not wait for it to be ready
   while(!mIE.Created) 
    Application.DoEvents();

   // Tell ie to 'navigate' to the webpage of interest, no popups or similar are supported, only html display code
   object arg1 = 0; object arg2 = ""; object arg3 = ""; object arg4 = "";
   mIE.Navigate(url,ref arg1,ref arg2, ref arg3, ref arg4);

   // Wait for the document to load
   while(!mDocComplete) 
    Application.DoEvents();

   // Wait for the DOM to be accessible 
   while(GetHtmlBody() == null)
    Application.DoEvents();

   // Capture whole the image
   Bitmap htmlimage=CaptureImage();
  	
  	
  	if (htmlimage!=null) {
  		
  		Bitmap pageBitmap=new Bitmap(htmlimage.Width,pageLength);
  		Graphics g=Graphics.FromImage(pageBitmap);
  		
  		int p=(int) Math.Floor(htmlimage.Height/pageLength);
  		if (p*pageLength<htmlimage.Height) p++;
  		string[] files=new string[p];
  		
  		
  		for (int i=0;i<files.GetLength(0);i++) {
  			g.Clear(Color.White);
  			g.DrawImage(htmlimage,new Rectangle(0,0,pageBitmap.Width,pageLength),new Rectangle(0,i*pageLength,pageBitmap.Width,pageLength),GraphicsUnit.Pixel);
  			files[i]=outputDir+outputFile+i+".png";
  			pageBitmap.Save(files[i],ImageFormat.Png);  			
  		}
  		
  		
  		htmlimage.Dispose();
  		g.Dispose();
  		pageBitmap.Dispose();
  		return files;
  	}
  	
   return null;
  }
  
  

  private void OnDocumentComplete(object sender, AxSHDocVw.DWebBrowserEvents2_DocumentCompleteEvent e)
  {
   mDocComplete = true;
  }

  private IHTMLElement2 GetHtmlBody()
  {
   IWebBrowser2 wb2 = (IWebBrowser2) mIE.GetOcx();
   IHTMLDocument2 htmlDocument2 = (IHTMLDocument2) wb2.Document;
   return (IHTMLElement2) 
	htmlDocument2.body;
  }

  private Bitmap CaptureImage()
  {

   try
   {
    // Figure out width and height of full page plus scroll bar slop
    IHTMLElement2 htmlBody2 = GetHtmlBody();
    int w = htmlBody2.scrollWidth;
    int h = htmlBody2.scrollHeight;

    // Make our embedded ie control that size
    if((mIE.Height != h + 30) || (mIE.Width != w + 30))
    {
     mIE.Height = h + 30;
     mIE.Width = w + 30;
    }

    // Setup bitmap memory and wrap a DC around it
    Bitmap bitmap = new Bitmap(w, h, PixelFormat.Format24bppRgb);
    Graphics graphics =  Graphics.FromImage(bitmap);    
    IntPtr memdc = graphics.GetHdc();
    IntPtr hbitmap = bitmap.GetHbitmap();
    SelectObject(memdc, hbitmap);

    // Tell ie to print into our dc/bitmap

    // Use PrintWindow windows api
    //   
 const uint PW_CLIENTONLY = 0x00000001;
    //   
 bool rv = PrintWindow(mIE.Handle, memdc, PW_CLIENTONLY);

    // Send it a print msg + flags
    const uint WM_PRINT =           0x0317;
//    const uint WM_PRINTCLIENT =  0x0318; 
//    const uint PRF_CHECKVISIBLE =   0x00000001;
    const uint PRF_NONCLIENT =      0x00000002;
    const uint PRF_CLIENT =         0x00000004;
    const uint PRF_ERASEBKGND =     0x00000008;
    const uint PRF_CHILDREN =       0x00000010;
    const uint PRF_OWNED =          0x00000020;
    int err = SendMessage(mIE.Handle, WM_PRINT, (uint)memdc, (uint) (PRF_CLIENT | PRF_NONCLIENT | PRF_OWNED | PRF_CHILDREN | PRF_ERASEBKGND));

    // Save image to disk as a PNG
    Bitmap bitmap2 = Bitmap.FromHbitmap(hbitmap);    

    // Cleanup
    DeleteObject(hbitmap);
    graphics.ReleaseHdc( memdc );
    graphics.Dispose();
    bitmap.Dispose();
   	
   	return bitmap2;

   }
   catch(Exception ex)
   {
    System.Console.WriteLine("Exception: " + ex.Message + "\n");
    System.Console.WriteLine(" source: " + ex.Source + "\n");
    System.Console.WriteLine(" stacktrace:" + ex.StackTrace + "\n");
    throw; // throwup
   }     
   
  }



  [DllImport("Gdi32.dll")]
  public static extern IntPtr SelectObject(
   IntPtr hdc,      // handle to DC
   IntPtr hgdiobj   // handle to object
   );

  [DllImport("Gdi32.dll")]
  public static extern bool DeleteObject(
   IntPtr hObject   // handle to graphic object
   );

  [DllImport("Gdi32.dll")]
  public static extern IntPtr 
CreateCompatibleDC(
   IntPtr hdc   // handle to DC
   );

  [DllImport("Gdi32.dll")]
  public static extern bool DeleteDC(
   IntPtr hdc   // handle to DC
   );

  [DllImport("User32.dll")]
  public static extern bool PrintWindow(
   IntPtr hwnd,               // Window to copy
   IntPtr hdcBlt,             // HDC to print into
   uint nFlags              // Optional flags  // must contain PW_CLIENTONLY
   );

  [DllImport("User32.dll")]
  public static extern int SendMessage(
   IntPtr hWnd,  // handle to destination window
   uint Msg,  // message
   uint wParam,  // firstmessage parameter
   uint lParam  // second message parameter
   );




  protected override void Dispose( bool disposing )
  {
   if( disposing )
   {
    if(components != null)
    {
     components.Dispose();
    }
   } base.Dispose( disposing );
  }

  private void InitializeComponent() {
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(Html2Image));
			this.mIE = new AxSHDocVw.AxWebBrowser();
			((System.ComponentModel.ISupportInitialize)(this.mIE)).BeginInit();
			this.SuspendLayout();
			// 
			// mIE
			// 
			this.mIE.Enabled = true;
			this.mIE.Location = new System.Drawing.Point(0, 0);
			//this.mIE.OcxState = ((System.Windows.Forms.AxHost.State)(resources.GetObject("mIE.OcxState")));
			this.mIE.Size = new System.Drawing.Size(680, 535);
			this.mIE.TabIndex = 7;
			this.mIE.DocumentComplete += new AxSHDocVw.DWebBrowserEvents2_DocumentCompleteEventHandler(this.OnDocumentComplete);
			// 
			// Html2Image
			// 
			this.Controls.AddRange(new System.Windows.Forms.Control[] {	this.mIE});
			this.Size = new System.Drawing.Size(292, 266);
			((System.ComponentModel.ISupportInitialize)(this.mIE)).EndInit();
			this.ResumeLayout(false);
		}
  

 }

